module nscrdll;

private import spfab.fabricate;

private import nscr.charset;
private import nscr.result;

private import std.c.windows.windows;
private import std.conv;
private import std.string;

// NscrLuaエクスポート関数
extern(C)
{
    bool NL_dostring (char *mem);
    void NL_register (char *funcname,int (*f)(void *));
    int NL_error(void *L,char *errorstr,...);
    void NL_settop(void *L,int index);
    int NL_gettop(void *L);
    int NL_isnumber(void *L,int index);
    int NL_istable(void *L,int index);
    int NL_isstring(void *L,int index);
    int NL_isboolean(void *L,int index);
    int NL_isnil(void *L,int index);
    double NL_tonumber(void *L,int index);
    int NL_tointeger(void *L,int index);
    char *NL_tostring(void *L,int index);
    int NL_toboolean(void *L,int index);
    void NL_pushnumber(void *L,double n);
    void NL_pushinteger(void *L,int n);
    void NL_pushstring(void *L,char *str);
    void NL_pushboolean(void *L,int b);
    void NL_gettable(void *L,int index);
    void NL_pop(void *L,int index);
}

// NScripterエクスポート関数

private bool initialized = false;

private void init(){
    if(initialized)
        return;
    initialized = true;
    char[256] fn;
    GetModuleFileNameA(null, cast(char*)fn, fn.sizeof);
    HMODULE hm = LoadLibraryA(cast(char*)fn);
    spfab.fabricate.init(hm);
    FreeLibrary(hm);
}

private const char[] VERSION = "0.3.0";

extern(C)
{
    // nsspfab_flipX(spnum)
    export int nsspfab_flipX(void *L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            if(argn != 1){
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_flipX(spnum)'"));
            }
            int spnum = NL_tointeger(L, 1);
            flipX(spnum);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 0;
    }

    // nsspfab_flipY(spnum)
    export int nsspfab_flipY(void *L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            if(argn != 1){
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_flipY(spnum)'"));
            }
            int spnum = NL_tointeger(L, 1);
            flipY(spnum);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 0;
    }

    // nsspfab_zoom(spnum, scaleX, scaleY[, autoresize])
    export int nsspfab_zoom(void *L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            if(argn != 3 && argn != 4){
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_zoom(spnum, scaleX, scaleY[, autoresize])'"));
            }
            int spnum = NL_tointeger(L, 1);
            double scaleX = NL_tonumber(L, 2);
            double scaleY = NL_tonumber(L, 3);
            bool autoresize = false;
            if(argn == 4 && !NL_isnil(L, 4)){
                autoresize = NL_toboolean(L, 4) != 0;
            }
            zoom(spnum, scaleX, scaleY, autoresize);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 0;
    }

    // nsspfab_rotate(spnum, degree[, autoresize])
    export int nsspfab_rotate(void *L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            if(argn != 2 && argn != 3){
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_rotate(spnum, degree[, autoresize])'"));
            }
            int spnum = NL_tointeger(L, 1);
            int degree = NL_tointeger(L, 2);
            bool autoresize = false;
            if(argn == 3 && !NL_isnil(L, 3)){
                autoresize = NL_toboolean(L, 3) != 0;
            }
            rotate(spnum, degree, autoresize);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 0;
    }
    
    // nsspfab_colorBy(spnum, dr, dg, db)
    export int nsspfab_colorBy(void *L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            if(argn != 4){
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_colorBy(spnum, dr, dg, db)'"));
            }
            int spnum = NL_tointeger(L, 1);
            int dr = NL_tointeger(L, 2);
            int dg = NL_tointeger(L, 3);
            int db = NL_tointeger(L, 4);
            colorBy(spnum, dr, dg, db);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 0;
    }
    
    // nsspfab_grayscale(spnum)
    export int nsspfab_grayscale(void *L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            if(argn != 1){
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_grayscale(spnum)'"));
            }
            int spnum = NL_tointeger(L, 1);
            grayscale(spnum);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 0;
    }
    
    // nsspfab_monocro(spnum[, threshold[, lowwhite]])
    export int nsspfab_monocro(void *L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            int threshold = 128;
            bool lowwhite = false;
            switch(argn){
            case 3:
                if(!NL_isnil(L, 3)) lowwhite = NL_toboolean(L, 3) != 0;
            case 2:
                if(!NL_isnil(L, 2)) threshold = NL_tointeger(L, 2);
            case 1:
                break;
            default:
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_monocro(spnum[, threshold[, lowwhite]])'"));
            }
            int spnum = NL_tointeger(L, 1);
            monocro(spnum, threshold, lowwhite);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 0;
    }
    
    // nsspfab_alternate(spnum)
    export int nsspfab_alternate(void* L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            if(argn != 1){
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_alternate(spnum)'"));
            }
            int spnum = NL_tointeger(L, 1);
            alternate(spnum);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 0;
    }
    
    // ptinopaque = nsspfab_ptinopaque(spnum, x, y[, cell])
    export int nsspfab_ptinopaque(void* L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            int x, y;
            int cell = 0;
            switch(argn){
            case 4:
                if(!NL_isnil(L, 4)) cell = NL_tointeger(L, 4);
            case 3:
                y = NL_tointeger(L, 3);
                x = NL_tointeger(L, 2);
                break;
            default:
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_ptinopaque(spnum, x, y[, cell])'"));
            }
            int spnum = NL_tointeger(L, 1);
            if(spnum < 0 || 999 < spnum){
                return NL_error(L, toNScrCharsetz("スプライト番号が不正です。0-999を指定してください。"));
            }
            if(cell < 0){
                return NL_error(L, toNScrCharsetz("セル番号が不正です。0以上を指定してください。"));
            }
            bool result = ptinopaque(spnum, x, y, cell);
            NL_pushboolean(L, result);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 1;
    }
    
    // success = nsspfab_blend(spnum, compnum, x, y[, operation])
    export int nsspfab_blend(void* L){
        try{
            init();
            int argn = NL_gettop(L); // 引数の個数を得る。
            int x, y;
            char[] operation;
            switch(argn){
            case 5:
                if(!NL_isnil(L, 5)) operation = toString(NL_tostring(L, 5));
            case 4:
                y = NL_tointeger(L, 4);
                x = NL_tointeger(L, 3);
                break;
            default:
                return NL_error(L, toNScrCharsetz("引数が不正です。'nsspfab_ptinopaque(spnum, x, y[, cell])'"));
            }
            int spnum = NL_tointeger(L, 1);
            int compnum = NL_tointeger(L, 2);
            if(spnum < 0 || 999 < spnum){
                return NL_error(L, toNScrCharsetz("スプライト番号が不正です。0-999を指定してください。"));
            }
            if(compnum < 0 || 999 < compnum){
                return NL_error(L, toNScrCharsetz("スプライト番号が不正です。0-999を指定してください。"));
            }
            bool result;
            switch(operation){
            case "add":
                result = blendAdd(spnum, compnum, x, y);
                break;
            case "sub":
                result = blendSub(spnum, compnum, x, y);
                break;
            default:
                result = blend(spnum, compnum, x, y);
            }
            NL_pushboolean(L, result);
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 1;
    }
    
    // version = nsspfab_version()
    export int nsspfab_version(void* L){
        try{
            NL_pushstring(L, toStringz(VERSION));
        }
        catch(Exception e){
            return NL_error(L, toNScrCharsetz(e.toString()));
        }
        return 1;
    }
}


extern(C){
    export int luaopen_nsspfab(void *L){
        NL_register("nsspfab_flipX", &nsspfab_flipX);
        NL_register("nsspfab_flipY", &nsspfab_flipY);
        NL_register("nsspfab_zoom", &nsspfab_zoom);
        NL_register("nsspfab_rotate", &nsspfab_rotate);
        NL_register("nsspfab_colorBy", &nsspfab_colorBy);
        NL_register("nsspfab_grayscale", &nsspfab_grayscale);
        NL_register("nsspfab_monocro", &nsspfab_monocro);
        NL_register("nsspfab_alternate", &nsspfab_alternate);
        NL_register("nsspfab_ptinopaque", &nsspfab_ptinopaque);
        NL_register("nsspfab_blend", &nsspfab_blend);
        NL_register("nsspfab_version", &nsspfab_version);
        return 0;
    }
}
