module spfab.matrix;

private import std.math;

private import spfab.vector2;

struct Matrix23{
    void setTranslation(ref Vector2 v){
        m00 = m11 = 1.0;
        m01 = m10 = 0.0;
        m02 = v.x;
        m12 = v.y;
    }
    
    void setRotation(int degree){
        double angle = degree * PI / 180.0;
        double cosval = cast(double)(cos(angle));
        double sinval = cast(double)(sin(angle));
        m00 = m11 = cosval;
        m01 = -sinval;
        m10 = sinval;
        m02 = m12 = 0.0;
    }
    
    void setScaling(ref Vector2 v){
        m00 = v.x;
        m11 = v.y;
        m01 = m10 = m02 = m12 = 0.0;
    }

    void opMulAssign(ref Matrix23 m){
        double tx, ty;
        tx = m00; ty = m01;
        m00 = tx * m.m00 + ty * m.m10;
        m01 = tx * m.m01 + ty * m.m11;
        m02 = tx * m.m02 + ty * m.m12 + m02;
        tx = m10; ty = m11;
        m10 = tx * m.m00 + ty * m.m10;
        m11 = tx * m.m01 + ty * m.m11;
        m12 = tx * m.m02 + ty * m.m12 + m12;
    }
    
    void multiply(out Vector2 dst, ref Vector2 src){
        double tx = src.x;
        dst.x = m00 * tx + m01 * src.y + m02; // LT * x + MT * y + RT * 1
        dst.y = m10 * tx + m11 * src.y + m12; // LB * x + MB * y + RB * 1
    }
private:
    double m00;
    double m01;
    double m02;
    double m10;
    double m11;
    double m12;
}


struct Matrix22{
    
    void setRotation(int degree){
        double angle = degree * PI / 180.0;
        double cosval = cast(double)(cos(angle));
        double sinval = cast(double)(sin(angle));
        m00 = m11 = cosval;
        m01 = -sinval;
        m10 = sinval;
    }
    
    void multiply(out Vector2 dst, Vector2 src){
        double tx = src.x;
        dst.x = m00 * tx + m01 * src.y; // LT * x + RT * y
        dst.y = m10 * tx + m11 * src.y; // LB * x + RB * y
    }
private:
    double m00;
    double m01;
    double m10;
    double m11;
}
